/**
 * nSX2 - an OpenSource PS2 Emulator
 * Copyright (C) 2002 Muad
 *
 * homepage: http://nsx2.emulation64.com
 * email address: atreides.muaddib@caramail.com
 * 
 * If you want to contribute to the project please contact
 * me first (maybe someone is already making what you are
 * planning to do).
 *
 *
 * This program is free software; you can redistribute it and/
 * or modify it under the terms of the GNU General Public Li-
 * cence as published by the Free Software Foundation; either
 * version 2 of the Licence, or any later version.
 *
 * This program is distributed in the hope that it will be use-
 * ful, but WITHOUT ANY WARRANTY; without even the implied war-
 * ranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public Licence for more details.
 *
 * You should have received a copy of the GNU General Public
 * Licence along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
**/
#include <stdlib.h>
#include <stdio.h>

#include "nSX2.h"
#include "PS2Edefs.h"


/*
TAG SOURCE CHAIN MODE : ID/FLAG
*/

#define	DMA_TRS_SRC_C_MODE_DSTS		0x00
#define	DMA_TRS_SRC_C_MODE_REFE		0x00

#define	DMA_TRS_SRC_C_MODE_CONT		0x10
#define	DMA_TRS_SRC_C_MODE_DST		0x10

#define	DMA_TRS_SRC_C_MODE_NEXT		0x20
#define	DMA_TRS_SRC_C_MODE_CALL		0x20

#define	DMA_TRS_SRC_C_MODE_REF		0x30
#define	DMA_TRS_SRC_C_MODE_REFS		0x40
#define	DMA_TRS_SRC_C_MODE_RET		0x60
#define	DMA_TRS_SRC_C_MODE_END		0x70


//Dma Transfert Size in Byte ( 16 = 128 bits )
#define DMA_TRS_TAG_SIZE    16



_u64 nb_dma_transfert = 0;

void		DmaUpdate() {
	
	_u32 	*DmaAdd;
	_u128 	temp;
	
	_u32 	boucle;
	_u64    Mask;
	

	_u16    Tag_ID_FLAG;

    _u32    CurrentTransfertAdd;
    _u8     DChainTransfertCompleted;
	
	
	/* if a memory write acces to the DMA Register is detected */
	if ( Ps2Bios.WriteToDmaArea == 1)  {
	
	if ( debug_dma == 1 )  printf("%8.8llx:DMA Acces ( write ): 0x1000%4.4llx h: ", MipsCpu.PC, Ps2Bios.DmaAreaWriteAt);
	
	switch ( Ps2Bios.DmaAreaWriteAt ) {
		
		
		/*Channel 2  - GIF*/
		case 0xA000: {
			if ( debug_dma == 1 ) printf("DmaChannel2_CHCRT = %llx\n",Ps2Bios.DmaAreaWriteVal);
			
			Ps2Bios.DmaC[ DMA_CH2 ].ctrl = Ps2Bios.DmaAreaWriteVal;		//store the contral value
			
			/* if the control value is START */
			if ( ( Ps2Bios.DmaC[ DMA_CH2 ].ctrl & 0x100 ) == 0x100 ) {
				
			
				switch ( Ps2Bios.DmaC[ DMA_CH2 ].ctrl & 0xFF ) {
			
			
				case 0x01 : {
					if ( debug_dma == 1 ) printf("GSgifTransfer 101 : Add:%16.16llx size:%ld\n", Ps2Bios.DmaC[DMA_CH2].memAdd, (_u32 ) Ps2Bios.DmaC[DMA_CH2].size);
				
					if ( Ps2Bios.DmaC[ DMA_CH2 ].memAdd > 0x01FFFFFF ) break;
				
					DmaAdd = (  _u32 * )	&MipsMemory [ Ps2Bios.DmaC[ DMA_CH2 ].memAdd ];	//compute the real addresse
					(*GSgifTransfer)( DmaAdd, (_u32 ) Ps2Bios.DmaC[ DMA_CH2 ].size) ;			// and pass the values to the GS pluggin
					
					break;
					}
			
				case 0x05 : {
				    if ( debug_dma == 1 ) printf("GSgifTransfer 105 : Add:%16.16llx \n", Ps2Bios.DmaC[DMA_CH2].tag);
			
				    DChainTransfertCompleted = FALSE;
				    CurrentTransfertAdd =  Ps2Bios.DmaC[ DMA_CH2 ].tag;
    				
    				while ( DChainTransfertCompleted != TRUE ) {
    				
                    MipsMemoryRead ( &temp  , DATA_QUAD , CurrentTransfertAdd , 0 ) ;
			
			        Ps2Bios.DmaC[ DMA_CH2 ].size         = temp.lo   & 0xFFFF;      //16 bits							
			        Tag_ID_FLAG                          = ( temp.lo >> 28 ) & 0x7  ;   
			        Ps2Bios.DmaC[ DMA_CH2 ].memAdd       = ( temp.lo >> 32 )& 0x7FFFFFFF; 
                 
                    switch ( Tag_ID_FLAG ) {
                    
                        case 0 :printf("DMA TRANSFERT TAG MODE refe NI\n");DChainTransfertCompleted = TRUE;break;
                             
                        case 1 : printf("DMA TRANSFERT TAG MODE cnt NI\n");DChainTransfertCompleted = TRUE;break;
  	                          
                        case 2 : printf("DMA TRANSFERT TAG MODE next NI\n");DChainTransfertCompleted = TRUE;break;
                        case 3 : printf("DMA TRANSFERT TAG MODE ref NI\n");DChainTransfertCompleted = TRUE;break;
                        case 4 : printf("DMA TRANSFERT TAG MODE refs NI\n");DChainTransfertCompleted = TRUE;break;
                        case 5 : printf("DMA TRANSFERT TAG MODE call NI\n");DChainTransfertCompleted = TRUE;break;
                        case 6 : printf("DMA TRANSFERT TAG MODE ret NI\n");DChainTransfertCompleted = TRUE;break;
                        case 7 : printf("DMA TRANSFERT TAG MODE end NI\n");DChainTransfertCompleted = TRUE;break;
                    }
                    
	                }
                
                
                
                
                
					break;
				
					}
			
				
				
				default: 	if ( debug_dma == 1 ) printf("GSgifTransfer Not Handled : %2.2llx \n", (Ps2Bios.DmaC[ DMA_CH2 ].ctrl & 0xFF )); break;
														
				}
				
				// This call write the end of the dma transfert
				// with writing 0 to the bit start of the DM2 channel Control
				Ps2Bios.DmaC[ DMA_CH2 ].ctrl = Ps2Bios.DmaC[ DMA_CH2 ].ctrl &  0xFFFFFEFF ;
				MipsMemoryWriteW ( SpecialAreaMemory,  Ps2Bios.DmaC[ DMA_CH2 ].ctrl , 0xa000 );	
				
				Ps2Bios.NeedUpdateScreen = Ps2Bios.NeedUpdateScreen + 1;
			
				nb_dma_transfert++;
				
							
			}
			
			
			
			
			break;
			}
		
		case 0xA010: {
			if ( debug_dma == 1 ) printf("DmaChannel2_MADR = %llx\n",Ps2Bios.DmaAreaWriteVal);
			Ps2Bios.DmaC[ DMA_CH2 ].memAdd = Ps2Bios.DmaAreaWriteVal;
			break;
		}
		
		case 0xA020: {
			if ( debug_dma == 1 ) printf("DmaChannel2_SIZE = %llx\n",Ps2Bios.DmaAreaWriteVal);
			Ps2Bios.DmaC[ DMA_CH2 ].size = Ps2Bios.DmaAreaWriteVal;
			break;
		}
		
		
		case 0xA030: {
			if ( debug_dma == 1 ) printf("DmaChannel2_TAG = %llx\n",Ps2Bios.DmaAreaWriteVal);
			Ps2Bios.DmaC[ DMA_CH2 ].tag = Ps2Bios.DmaAreaWriteVal;
			break;
		}
		
		case 0xA040: {
			if ( debug_dma == 1 ) printf("DmaChannel2_ASR0 = %llx\n",Ps2Bios.DmaAreaWriteVal);
			Ps2Bios.DmaC[ DMA_CH2 ].asr0 = Ps2Bios.DmaAreaWriteVal;
			break;
		}
		
		case 0xA050: {
			if ( debug_dma == 1 ) printf("DmaChannel2_ASR1 = %llx\n",Ps2Bios.DmaAreaWriteVal);
			Ps2Bios.DmaC[ DMA_CH2 ].asr1 = Ps2Bios.DmaAreaWriteVal;
			break;
		}
		
		case 0xA080: {
			if ( debug_dma == 1 ) printf("DmaChannel2_SADR = %llx\n",Ps2Bios.DmaAreaWriteVal);
			Ps2Bios.DmaC[ DMA_CH2 ].saddr = Ps2Bios.DmaAreaWriteVal;
			break;
		}
		
		
		
		/*********************************************************************************/
		/*Channel 5  - SIF 0*/
		case 0xC000: {
			if ( debug_dma == 1 ) printf("DmaChannel5_CHCR = %llx\n",Ps2Bios.DmaAreaWriteVal);
			Ps2Bios.DmaC[ DMA_CH5 ].ctrl = Ps2Bios.DmaAreaWriteVal;		//store the control value
			
								
			if ( (Ps2Bios.DmaC[ DMA_CH5 ].ctrl & 0x100 ) == 0x100 ) {
				
				if ( Ps2Bios.DmaC[ DMA_CH5 ].memAdd != 0 ) {
		
					DmaPrintChannelInfo ( DMA_CH5 );
		
				}
					
		
		
				// This call write the end of the dma transfert
				// with writing 0 to the bit start of the DM2 channel Control
				Ps2Bios.DmaC[ DMA_CH5 ].ctrl = Ps2Bios.DmaC[ DMA_CH5 ].ctrl &  0xFFFFFEFF ;
				MipsMemoryWriteW ( SpecialAreaMemory,  Ps2Bios.DmaC[ DMA_CH5 ].ctrl , 0xc000 );	
				
			}
			
			
			
			break;
		}
		
		
		case 0xC010: {
			if ( debug_dma == 1 ) printf("DmaChannel5_MADR = %llx\n",Ps2Bios.DmaAreaWriteVal);
			Ps2Bios.DmaC[ DMA_CH5 ].memAdd = Ps2Bios.DmaAreaWriteVal;		//store the control value
			
			break;
		}
		
		
		
		case 0xC020: {
			if ( debug_dma == 1 ) printf("DmaChannel5_QWC = %llx\n",Ps2Bios.DmaAreaWriteVal);
			Ps2Bios.DmaC[ DMA_CH5 ].size = Ps2Bios.DmaAreaWriteVal;		//store the control value
			
			break;
		}
		
		
		/*********************************************************************************/
		/*DMA Channel Controller*/
		case 0xE010: {
			if ( debug_dma == 1 ) printf("DMAC Interrup Status = %llx\n",Ps2Bios.DmaAreaWriteVal);
			Mask = 0x10000;
			
		    for ( boucle = 0 ; boucle < 10 ; boucle++ ) {
		        
		        if (( Ps2Bios.DmaAreaWriteVal & Mask ) == Mask ) Ps2Bios.DmaH[ boucle ].Enable = TRUE;
		        else Ps2Bios.DmaH[ boucle ].Enable = FALSE;
		        Mask = Mask << 1;
		    }
		    
		    /*printf("DMA Interrupt CH5 : %d\n",Ps2Bios.DmaH[ DMA_CH5 ].Enable );*/
			
			break;
		}
		
		
		
		
		default: 	printf("DMA Adress Non-Impl.  1000%4llx h (i:%lld) \n", Ps2Bios.DmaAreaWriteAt,MipsCpu.Nb_instructions);
				
				
	
	}
	
	/* the DMA Register access is DONE */
	Ps2Bios.WriteToDmaArea = 0;
	}


	/* if a memory read acces to the DMA Register is detected */
	if ( Ps2Bios.ReadToDmaArea == 1)  {
	
		if ( debug_dma == 1 )  printf("%8.8llx:DMA Acces ( read  ):1000 %4llx h: %llx\n", MipsCpu.PC, Ps2Bios.DmaAreaReadAt,Ps2Bios.DmaAreaReadVal);
		
		Ps2Bios.ReadToDmaArea = 0;
	}
	
	
	
	 if ( Ps2Bios.Sif[ 0 ].Enable == 1 ) {

	 	printf("DMA:Sif_0 : Start\n");
	 	printf("|DmaHandler Add: %8.8llx\n",Ps2Bios.DmaH[ 5 ].Add);
	 	
	 	
	 	
	 	Ps2Bios.DmaH[ 5 ].ReturnAdd = MipsCpu.PC;
	 	Ps2Bios.DmaH[ 5 ].Running = 1;
	 	MipsCpu.PC = Ps2Bios.DmaH[ 5 ].Add;
	 	
	 	
	 	Ps2Bios.Sif[ 0 ].Enable = 0;
	 	
	 	//debug_op = 1;
	 	
	}
	
	
	
	
		
		
}


void DmaPrintChannelInfo ( _u8 fChannel ) {
	

	printf("\nDMA Channel n %2.2d\n",fChannel);
	printf("Channel Control register: %llx\n",Ps2Bios.DmaC[fChannel].ctrl);
	printf("Memory address          : %llx\n",Ps2Bios.DmaC[fChannel].memAdd);
	printf("Quadra-word counter     : %llx\n",Ps2Bios.DmaC[fChannel].size);
	printf("Tag address             : %llx\n",Ps2Bios.DmaC[fChannel].tag);
	printf("Assembler Reserved 0    : %llx\n",Ps2Bios.DmaC[fChannel].asr0);
	printf("Assembler Reserved 1    : %llx\n",Ps2Bios.DmaC[fChannel].asr1);
	printf("???                     : %llx\n",Ps2Bios.DmaC[fChannel].saddr);
	
	
}

