/**
 * nSX2 - an OpenSource PS2 Emulator
 * Copyright (C) 2002 Muad
 *
 * homepage: http://nsx2.emulation64.com
 * email address: atreides.muaddib@caramail.com
 * 
 * If you want to contribute to the project please contact
 * me first (maybe someone is already making what you are
 * planning to do).
 *
 *
 * This program is free software; you can redistribute it and/
 * or modify it under the terms of the GNU General Public Li-
 * cence as published by the Free Software Foundation; either
 * version 2 of the Licence, or any later version.
 *
 * This program is distributed in the hope that it will be use-
 * ful, but WITHOUT ANY WARRANTY; without even the implied war-
 * ranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public Licence for more details.
 *
 * You should have received a copy of the GNU General Public
 * Licence along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
**/
#include <stdio.h>
#include <stdlib.h>

#include "nSX2.h"
#include "elf_file.h"

#include "PS2Edefs.h"


_u8	* MipsMemory;
_u8	* SpecialAreaMemory;
_u8	* GSAreaMemory;
_u8 * BiosMemory;


/****************************************************************************************************/


/*
0x00100000-0x01ffffff RAM Mirror Address. (cached)
0x20100000-0x21ffffff RAM Mirror Adress. (not cached)
0x30100000-0x31ffffff RAM Mirror Adress. (not cached & accelerated)
*/
_u8 MipsMemoryAlloc( void )  {
	_u8 	Result;
	char tempText[100];	
	
	
	MipsMemory = malloc ( sizeof( _u8 ) * 0x01FFFFFF );
	if (MipsMemory != NULL ) {
		sprintf(tempText,"R5900:Allocation Main Memory 32 Mbits...OK" );	
		Result = TRUE;
	}
	else {
		sprintf(tempText,"R5900:Allocation Main Memory 32 Mbits...KO" );	
		Result = FALSE;
	}
	
	PrintToEmuOutput(tempText);	

	return Result;
}


/*
0x10000000 - 0x1000ffff

COUNTERS' REGISTERS
IPU REGISTERS (MPEG2 DECODER)
GIF REGISTERS
VIF0 REGISTERS
VIF1 REGISTERS
FIFO
DMA CH0 REGISTERS (Linked to VIF0)
DMA CH1 REGISTERS (Linked to VIF1)
DMA CH2 REGISTERS (Linked to GIF)
DMA CH3 REGISTERS (Linked to IPU (FROM???))
DMA CH4 REGISTERS (Linked to IPU (TO???))
DMA CH5 REGISTERS (Linked to SIF0)
DMA CH6 REGISTERS (Linked to SIF1)
DMA CH7 REGISTERS (Linked to SIF2)
DMA CH8 REGISTERS (Linked to SPR  (form SCRATCH PAD to RAM???)
DMA CH9 REGISTERS (Linked to SPR  (form RAM to SCRATCH PAD???)
DMA CONTROL REGISTERS
*/
_u8 MipsSpecialAreaMemoryAlloc( void )  {
	_u8 	Result;	
	char tempText[100];	
    SpecialAreaMemory	 = NULL;
	SpecialAreaMemory = malloc ( sizeof( _u8 ) * 0xFFFF );
	if (SpecialAreaMemory != NULL ) {
		sprintf(tempText,"R5900:Allocation Special Memory 64 kbytes... OK" );	
		Result = TRUE;
	}
	else {
		sprintf(tempText,"R5900:Allocation Special Memory 64 kbytes... KO" );	
		Result = FALSE;
	}

	PrintToEmuOutput(tempText);	

	return Result;
}

/*
0x12000000 - 0x120010C0
*/
_u8 MipsGSAreaMemoryAlloc( void )  {
	_u8 	Result;	
	char tempText[100];	
	
    GSAreaMemory = NULL;
	GSAreaMemory = malloc ( sizeof( _u8 ) * 0x10C0 );
	if (GSAreaMemory != NULL ) {
		sprintf(tempText,"R5900:Allocation GS Register Memory 4288 bytes... OK" );	
		Result = TRUE;
	}
	else {
		sprintf(tempText,"R5900:Allocation GS Register Memory 4288 bytes... KO" );	
		Result = FALSE;
	}
	
	PrintToEmuOutput(tempText);	
	
	return Result;
}



/*
0x1FC00000 - 0x1FFFFFFF ROM BIOS Mirror Adress. (not cached)
0x9FC00000 - 0x9FFFFFFF ROM BIOS Mirror Adress. (cached)
0xBFC00000 - 0xBFFFFFFF ROM BIOS Mirror Adress. (not cached)
*/
_u8 MipsBiosMemoryAlloc( void )  {
	_u8 	Result;	
	char tempText[100];
	
	BiosMemory = malloc ( sizeof( _u8 ) * 0x3FFFFF );
	if (BiosMemory != NULL ) {
		sprintf(tempText,"R5900:Allocation BIOS Memory 4 M bytes... OK ");
		Result = TRUE;
	}
	else {
		sprintf(tempText,"R5900:Allocation BIOS Memory 4 M bytes... KO ");
		Result = FALSE;
	}
	
	PrintToEmuOutput(tempText);	
	
	return Result;
}

void	freeMemoryPrefetchOpcode ( void ) {

	_u32	MemorySize;
	_u32	boucle;


		if ( previous_INTERPRETER_MODE == 1 ) {

			MemorySize =   ( MipsCpu.OpcodeEndAdd[MipsCpu.NbOfExecutableSection] - MipsCpu.OpcodeStartAdd[MipsCpu.NbOfExecutableSection]) >> 2  ;

			
			for ( boucle = 0 ; boucle < MemorySize ; boucle++ )	{
				if ( MipsCpu.ListPrefetchedOpcode[boucle] != NULL ) free ( MipsCpu.ListPrefetchedOpcode[boucle] ) ;
			}
			
				
		} /*else if ( MipsCpu.ListPrefetchedOpcode[0] != NULL ) free ( MipsCpu.ListPrefetchedOpcode[0] ) ;*/
	
		if ( MipsCpu.ListPrefetchedOpcode != NULL ) free ( MipsCpu.ListPrefetchedOpcode );
	
	
	
}



void FreeAllMemory( void ) {

	
	if ( MipsMemory != NULL ) 			free(MipsMemory );
	if ( SpecialAreaMemory != NULL ) 	free(SpecialAreaMemory);
	if ( GSAreaMemory != NULL ) 		free(GSAreaMemory);
	if ( BiosMemory != NULL ) 			free(BiosMemory);
	freeMemoryPrefetchOpcode();
	
	//sprintf(tempText,"FreeMemory: Main - SpecialArea - GS Register - Bios");
	//PrintToEmuOutput(tempText);	
	
}

/****************************************************************************************************/


void CopyBiosToMemory ( void  ) {

		_u64 Addr;
		char c;
		char tempText[100];
				
		Addr = 0;
		
		fseek( BiosFile, 0, SEEK_SET );
		while ( fread( &c , sizeof( char ), sizeof( char ), BiosFile ) != 0 ) {  
			BiosMemory[Addr] = c;
			Addr++;
		}
		
		sprintf(tempText,"Init:Copy BIOS To Memory:...Done");
		PrintToEmuOutput(tempText);		

}


void CopyProgramToMemory( void ) {
	
	_u64 Addr;
	_u64 fIndex;
	_u64 Index2;
	_u64 ReadError;
	char c;
	char tempText[100];	
	
	sprintf(tempText,"Init:Copy Program To Memory: ");
	PrintToEmuOutput(tempText);	
	
	ReadError = 0;
	
	
	MipsCpu.NbOfExecutableSection = 0;
	
	
	for ( Index2 = 0 ; Index2 <  ico_elf_hdr.num_of_prg_hdr ; Index2++ ) {
		
		
		
		Addr = ico_elf_prg_hdr[Index2].physic_add_of_segment;
		MipsCpu.OpcodeStartAdd[MipsCpu.NbOfExecutableSection] = ico_elf_prg_hdr[Index2].physic_add_of_segment;
		
		
		fseek( ElfFile,  ico_elf_prg_hdr[Index2].offs_from_start_to_prg_seg , SEEK_SET );	
	
	
		
		for ( fIndex = 0 ; fIndex < ico_elf_prg_hdr[Index2].num_of_bytes_in_file_ima ; fIndex++ ) {
	
			if ( fread( &c , sizeof( char ), sizeof( char ), ElfFile ) != 0 ) {
				MipsMemory[Addr] = c;
				
			} else ReadError++;
			
			Addr++;
		}
		
		
		MipsCpu.OpcodeEndAdd[MipsCpu.NbOfExecutableSection] = MipsCpu.OpcodeStartAdd[MipsCpu.NbOfExecutableSection] + ico_elf_prg_hdr[Index2].num_of_bytes_in_file_ima;
	
		MipsCpu.NbOfExecutableSection = MipsCpu.NbOfExecutableSection + 1;
		
	}
	
	if (ReadError != 0 )	sprintf(tempText,"Init:Copy Program To Memory: Done with %lld errors",ReadError);
	else	sprintf(tempText,"Init:Copy Program To Memory: Done");
	
	PrintToEmuOutput(tempText);	

}

void CopySectionToMemory ( void ) {

	_u64 Addr = 0;
	_u8 index;
	_u64 size;
	_u64 ReadError;	
	char c;
	char tempText[100];	
	
		
	sprintf(tempText,"Init:Copy Section To Memory: ");
	PrintToEmuOutput(tempText);	

	ReadError = 0;
		
	for ( index = 0 ; index <  ico_elf_hdr.num_of_sec_hdr ; index++ ) {

			
			if ( ( ico_elf_sec_hdr[index].sect_start_add +  ico_elf_sec_hdr[index].size ) < 0x01FFFFFF ) {
	   
                if ( ico_elf_sec_hdr[index].type != 8 ) {
       			
				Addr =  ico_elf_sec_hdr[index].sect_start_add,
				fseek( ElfFile,   ico_elf_sec_hdr[index].offset_from_start  , SEEK_SET );	
			
				for ( size = 0 ; size < ico_elf_sec_hdr[index].size ; size++ ) {
				
					if ( fread( &c , sizeof( char ), sizeof( char ), ElfFile ) != 0 ) {
						MipsMemory[Addr] = c;
						
					} else ReadError++;
					
					Addr++;
				}
          }
            else {
            
            	for ( size = 0 ; size < ico_elf_sec_hdr[index].size ; size++ ) {
			     	MipsMemory[Addr] = 0;
					Addr++;
				}
            }
                    
   
            }
			else 
			{
			    printf("Section %d Too Long\n", index);
			
				ReadError++;
			}
			
			
	}


	if (ReadError != 0 ) sprintf(tempText,"Init:Copy Section To Memory: Done with %lld errors",ReadError);
						
	else sprintf(tempText,"Init:Copy Section To Memory: Done");
	
	PrintToEmuOutput(tempText);

}

/***********************************************************************************************/

void	MipsMemoryWrite ( _u128 fVal  , _u8 fDataType , _u64 fBase , int fOffset ) {
	
	_u8	Mem_Perform;
	_u64	PhysicalAdd;
	
	_u8 * MemTarget;

	char tempText[100];	
	
	Mem_Perform = 0;
	PhysicalAdd = ( fBase + fOffset ) & 0xFFFFFFFF;
	
	
	//	0x10000000 - 0x1000FFFF	
	if ( ( PhysicalAdd >= 0x10000000 ) && ( PhysicalAdd <= 0x1000FFFF ) ){
		MemTarget = SpecialAreaMemory;
		MipsSpecialMemWrite( MemTarget , fVal ,  fDataType , fBase , fOffset );
		Mem_Perform = 1;
		return;
	}


	//	0x12000000 - 0x12001080	
	if ( ( PhysicalAdd >= 0x12000000 ) && ( PhysicalAdd <= 0x12001080 ) ){
		MemTarget = GSAreaMemory;
		MipsGSAreaMemWrite( MemTarget, fVal ,  fDataType , fBase , fOffset );
		Mem_Perform = 1;
		return;
	}

	
	
	//	0x80000000 - 0x9FFFFFFF			//KSEG0
	if  (( PhysicalAdd >= 0x80000000 ) && ( PhysicalAdd < 0x9FFFFFFF ) ) {
		//	if (PhysicalAdd >=  0x81effc70 ) printf("Mem:WRITE %16.16llx PC:%8.8lx\n",PhysicalAdd, MipsCpu.PC);
		PhysicalAdd = PhysicalAdd & 0x01FFFFFF;
	}
	
	//	0x00100000 - 0x01FFFFFF
	//	0x20100000 - 0x21FFFFFF
	//	0x30100000 - 0x31FFFFFF
	if (	(( PhysicalAdd >= 0x00100000 ) && ( PhysicalAdd < 0x01FFFFFF )) ||
			(( PhysicalAdd >= 0x20100000 ) && ( PhysicalAdd < 0x21FFFFFF )) ||
			(( PhysicalAdd >= 0x30100000 ) && ( PhysicalAdd < 0x31FFFFFF )) )
	 {
		
		PhysicalAdd = PhysicalAdd &	0x01FFFFFF;

		MemTarget = MipsMemory;
		
		switch ( fDataType ) {
			case DATA_QUAD:		MipsMemoryWriteDW 	( MemTarget, fVal.lo, PhysicalAdd );
								MipsMemoryWriteDW 	( MemTarget, fVal.hi, PhysicalAdd + 8 );break;

			case DATA_DW:		MipsMemoryWriteDW	( MemTarget, fVal.lo, PhysicalAdd );break;

			case DATA_DWH:		MipsMemoryWriteDW	( MemTarget, fVal.hi, PhysicalAdd );break;
			
			case DATA_W:		MipsMemoryWriteW	( MemTarget, fVal.lo, PhysicalAdd );break;

			case DATA_HW:		MipsMemoryWriteHW	( MemTarget, fVal.lo, PhysicalAdd );break;

			case DATA_B:		MipsMemoryWriteB	( MemTarget, fVal.lo, PhysicalAdd );break;
		
			default:			sprintf(tempText,"Mem:Write DataType Not Handled\n");
								PrintToEmuOutput(tempText);								
								MipsCpu.halt = 1;break;
							
		}
		Mem_Perform = 1;
		return;
	}

	// Others
	if ( Mem_Perform == 0 ) {
		sprintf(tempText,"Mem:Write:NEED SPECIAL HANDLE ADD :  %llx\n",PhysicalAdd );
		PrintToEmuOutput(tempText);	
		//MipsCpu.halt = 1;
	}


}


void MipsMemoryWriteDW ( _u8 * Mem ,  _u64 fVal , _u64 fAdd ) {
/*
	_u64 *pMem64;
	_u8  *pMem8;
	pMem8 = (Mem + fAdd);
	pMem64 = ( _u64 *)pMem8;
	*pMem64 = (_u64)fVal;
*/

	(_u64)*(( _u64 *)(Mem + fAdd)) = (_u64)fVal;

}

void MipsMemoryWriteW  (_u8 * Mem ,  _u64 fVal , _u64 fAdd ) {
/*	
	_u32 *pMem32;
	_u8  *pMem8;
	pMem8 = (Mem + fAdd);
	pMem32 = ( _u32 *)pMem8;
	*pMem32 = (_u32)(fVal & 0xFFFFFFFF);		
*/	
	
	(_u32)*(( _u32 *)(Mem + fAdd)) = (_u32)(fVal & 0xFFFFFFFF);	

}

void MipsMemoryWriteHW  ( _u8 * Mem , _u64 fVal , _u64 fAdd ) {
/*
	_u16 *pMem16;
	_u8  *pMem8;
	pMem8 = (Mem + fAdd);
	pMem16 = ( _u16 *)pMem8;
	*pMem16 = (_u16)(fVal & 0xFFFF);		
*/
	(_u16)*(( _u16 *)(Mem + fAdd)) = (_u16)(fVal & 0xFFFF);

}

void MipsMemoryWriteB  ( _u8 * Mem ,_u64 fVal , _u64 fAdd ) {
/*
	_u8  *pMem8;
	pMem8 = (Mem + fAdd);
	*pMem8 = (_u8)(fVal & 0xFF);		
*/
	(_u8)*(Mem + fAdd) = (_u8)(fVal & 0xFF);
}

/******************************************************************************************/

void	MipsMemoryRead ( _u128 * fVal  , _u8 fDataType , _u64 fBase , int fOffset ) {

	_u8	Mem_Perform;
	_u64	PhysicalAdd;
	
	_u8 * MemTarget;
	
	Mem_Perform = 0;
	PhysicalAdd = ( fBase + fOffset ) & 0xFFFFFFFF;
	
	
	//	0x10000000 - 0x1000FFFF	
	if ( ( PhysicalAdd >= 0x10000000 ) && ( PhysicalAdd < 0x1000FFFF ) ){
		MemTarget = SpecialAreaMemory;
		MipsSpecialMemRead(MemTarget , fVal ,  fDataType , fBase , fOffset );
		Mem_Perform = 1;
		return;
	}

	//	0x12000000 - 0x12001080	
	if ( ( PhysicalAdd >= 0x12000000 ) && ( PhysicalAdd <= 0x12001080 ) ){
		MemTarget = GSAreaMemory;
		MipsGSAreaMemRead( MemTarget,  fVal ,  fDataType , fBase , fOffset );
		Mem_Perform = 1;
		return;
	}

	// 0x1FC00000 - 0x1FFFFFFF	ROM BIOS Mirror Adress.	(not cached)
	// 0x9FC00000 - 0x9FFFFFFF	ROM BIOS Mirror Adress.	(cached)
	// 0xBFC00000 - 0xBFFFFFFF	ROM BIOS Mirror Adress.	(not cached)
	if ( 	(( PhysicalAdd >= 0x1FC00000 ) && ( PhysicalAdd < 0x1FFFFFFF )) ||
	 		(( PhysicalAdd >= 0x9FC00000 ) && ( PhysicalAdd < 0x9FFFFFFF )) ||
	 		(( PhysicalAdd >= 0xBFC00000 ) && ( PhysicalAdd < 0xBFFFFFFF )) )
	 {

		MemTarget = BiosMemory; 
	 	MipsBiosAreaMemRead(  MemTarget , fVal ,  fDataType , fBase , fOffset );
		Mem_Perform = 1;
		return;
	 
	 }


	// 0x80000000 - 0x9FFFFFFF			//KSEG0
	if  (( PhysicalAdd >= 0x80000000 ) && ( PhysicalAdd < 0x9FFFFFFF ) ) {
	//	if (PhysicalAdd >=  0x81effc70 ) printf("Mem:READ  %16.16llx PC:%8.8lx\n",PhysicalAdd, MipsCpu.PC);
		PhysicalAdd = PhysicalAdd & 0x01FFFFFF;

	}


	//	0x00100000 - 0x01FFFFFF
	//	0x20100000 - 0x21FFFFFF
	//	0x30100000 - 0x31FFFFFF
	if ( 	(( PhysicalAdd >= 0x00100000 ) && ( PhysicalAdd < 0x01FFFFFF )) ||
	 		(( PhysicalAdd >= 0x20100000 ) && ( PhysicalAdd < 0x21FFFFFF )) ||
	 		(( PhysicalAdd >= 0x30100000 ) && ( PhysicalAdd < 0x31FFFFFF )) )
	 {
		
		PhysicalAdd = PhysicalAdd &	0x01FFFFFF;
		MemTarget = MipsMemory; 
		switch ( fDataType ) {
		
			case DATA_QUAD:		MipsMemoryReadDQ 	( MemTarget, fVal, PhysicalAdd );break;
			
			//case DATA_DWH:		MipsMemoryReadDWH	( MemTarget, fVal, PhysicalAdd );break;
			
			case DATA_DW:		MipsMemoryReadDW	( MemTarget, fVal, PhysicalAdd );break;
			
			case DATA_W:		MipsMemoryReadW		( MemTarget, fVal, PhysicalAdd );break;

			case DATA_HW:		MipsMemoryReadHW	( MemTarget, fVal, PhysicalAdd );break;

			case DATA_B:		MipsMemoryReadB		( MemTarget, fVal, PhysicalAdd );break;

			default:			printf("Mem:Read DataType Not Handled\n");
								MipsCpu.halt = 1;break;
					
		}
		Mem_Perform = 1;
		return;
	}

	// Others
	if ( Mem_Perform == 0 ) {
		printf("Mem:Read:NEED SPECIAL HANDLE ADD :  %llx\n",PhysicalAdd );
		//MipsCpu.halt = 1;
	}

}

void MipsMemoryReadDQ ( _u8 * Mem,  _u128 * fVal , _u64 fAddr ) {
/*
	_u64 *pMem64;
	_u8  *pMem8;
	
	pMem8 = (Mem + fAddr );
	pMem64 = ( _u64 *)pMem8;
	fVal->lo = *pMem64;
	
		
	pMem8 = (Mem + fAddr + 8);
	pMem64 = ( _u64 *)pMem8;
	fVal->hi = *pMem64;
*/	
	fVal->lo = *(( _u64 *)(Mem + fAddr));
	fVal->hi = *(( _u64 *)(Mem + fAddr + 8));
	
}




void MipsMemoryReadDW ( _u8 * Mem,  _u128 * fVal , _u64 fAddr ) {
/*
	_u64 *pMem64;
	_u8  *pMem8;
	
	pMem8 = (Mem + fAddr);
	pMem64 = ( _u64 *)pMem8;

	fVal->lo = *pMem64;
*/

	fVal->lo = *(( _u64 *)(Mem + fAddr));
	
}

void MipsMemoryReadDWH ( _u8 * Mem,  _u128 * fVal , _u64 fAddr ) {

	_u64 *pMem64;
	_u8  *pMem8;
	
	pMem8 = (Mem + fAddr);
	pMem64 = ( _u64 *)pMem8;

	fVal->hi = *pMem64;
	
}


void MipsMemoryReadW ( _u8 * Mem, _u128 * fVal , _u64 fAddr ) {
/*
	_u32 *pMem32;
	_u8  *pMem8;
	
	pMem8 = (Mem + fAddr);
	pMem32 = ( _u32 *)pMem8;

	fVal->lo = *pMem32;
*/

	
	fVal->lo = *(( _u32 *)(Mem + fAddr));
}

void MipsMemoryReadHW ( _u8 * Mem,  _u128 * fVal , _u64 fAddr ) {
/*
	_u16 *pMem16;
	_u8  *pMem8;
	
	pMem8 = (Mem + fAddr);
	pMem16 = ( _u16 *)pMem8;

	fVal->lo = *pMem16;
*/	
	
	
	fVal->lo = *(( _u16 *)(Mem + fAddr));
	
}


void MipsMemoryReadB ( _u8 * Mem,  _u128 * fVal , _u64 fAddr ) {

/*
	_u8  *pMem8;
	pMem8 = (Mem + fAddr);
	fVal->lo = *pMem8;
*/

	fVal->lo = *(Mem + fAddr);
	
}


/*****************************************************************************************************/

void  MipsSpecialMemRead( _u8 * Mem, _u128 * fVal  , _u8 fDataType , _u64 fBase , int fOffset ) {

	_u64 RealAdd;
	RealAdd = ( fBase + fOffset ) & 0xFFFF;

	switch ( fDataType ) {
					
		case DATA_DW:	MipsMemoryReadDW	( Mem, fVal, RealAdd );break;
		
		case DATA_W:	MipsMemoryReadW 	( Mem, fVal, RealAdd );break;

		default: 		printf("MemSpecial:Read DataType Not Handled\n");
						MipsCpu.halt = 1;break;
	}


	if ( RealAdd >= 0x8000 ) {
		Ps2Bios.ReadToDmaArea = 1;
		Ps2Bios.DmaAreaReadAt = RealAdd;
		Ps2Bios.DmaAreaReadVal = fVal->lo;
	}

}


void  MipsSpecialMemWrite( _u8 * Mem , _u128  fVal  , _u8 fDataType , _u64 fBase , int fOffset ) {

	_u64 RealAdd;
	RealAdd = ( fBase + fOffset ) & 0xFFFF;

	


	/* Can't write to DMA Status */
	//if ( RealAdd == 0xE010 ) 		return;	
	

	switch ( fDataType ) {
				
		case DATA_DW:	MipsMemoryWriteDW	( Mem, fVal.lo, RealAdd );break;
				
		case DATA_W:	MipsMemoryWriteW	( Mem, fVal.lo, RealAdd );break;
										
		default:		printf("MemSpecial:Write DataType Not Handled\n");
						MipsCpu.halt = 1;break;
	}


	if ( RealAdd >= 0x8000 ) {
		Ps2Bios.WriteToDmaArea = 1;
		Ps2Bios.DmaAreaWriteAt = RealAdd;
		Ps2Bios.DmaAreaWriteVal = fVal.lo;
		
		DmaUpdate();
	}


}



/***********************************************************************************************/



void  MipsGSAreaMemWrite( _u8 * Mem , _u128  fVal  , _u8 fDataType , _u64 fBase , int fOffset ) {

	_u32 Add;

	Add = ( fBase + fOffset ) ;
	
	switch ( fDataType ) {
			
		case DATA_DW:	if ( debug_gs == 1 ) printf("%8.8llx: GSwrite64 at %lx with %llx\n",MipsCpu.PC,Add,fVal.lo);
						(*GSwrite64)	( Add, fVal.lo );break;

		case DATA_W:	if ( debug_gs == 1 ) printf("%8.8llx: GSwrite32 at %lx with %llx\n",MipsCpu.PC,Add,fVal.lo);
						(*GSwrite32)	( Add, (_u32) (fVal.lo&0x00000000FFFFFFFF) );break;
			
		default:		printf("MemGSArea:Write DataType Not Handled\n");
						MipsCpu.halt = 1;break;

	}
}



void MipsGSAreaMemRead( _u8 * Mem, _u128 * fVal  , _u8 fDataType , _u64 fBase , int fOffset ) {

	_u64 RealAdd;
	RealAdd = ( fBase + fOffset ) ; //& 0xFFFF;


    if ( RealAdd == 0x12001000 ) {
    
        fVal->lo = 0x8;
        return;

    }


	switch ( fDataType ) {

				
		case DATA_DW:	fVal->lo = (*GSread64)((_u32)RealAdd) ;
						if ( debug_gs == 1 ) printf("%8.8llx: GSread64 at %llx with %llx\n",MipsCpu.PC,RealAdd,fVal->lo);
						break;

		case DATA_W:	fVal->lo = (*GSread32)((_u32)RealAdd) ;
						if ( debug_gs == 1 ) printf("%8.8llx: GSread32 at %llx with %llx\n",MipsCpu.PC,RealAdd,fVal->lo);
						break;
	
		default:		printf("MemGSArea:Read DataType Not Handled\n");
						MipsCpu.halt = 1;break;
		}

}

/*****************************************************************************************************/


void  MipsBiosAreaMemRead(_u8 * Mem, _u128 * fVal  , _u8 fDataType , _u64 fBase , int fOffset ) {

	_u64 RealAdd;
	RealAdd = ( fBase + fOffset ) & 0x3FFFFF;

	switch ( fDataType ) {

		case DATA_W:	MipsMemoryReadW	( Mem, fVal, RealAdd );break;

		case DATA_B:	MipsMemoryReadB ( Mem, fVal, RealAdd );break;
												
		default:		printf("MemBIOS:Read DataType Not Handled\n");
						MipsCpu.halt = 1;break;
	}

}



